var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { join, isAbsolute } from 'path';
import validateNpmPackageName from 'validate-npm-package-name';
import { isRelativePath, installDeps, logger } from "../../utils";
import { Template, GitRepository, NpmRepository, LocalRepository, Effect } from "../../internal";
import semver from 'semver';
export class Repository {
    constructor() {
        this.owner = '';
        this.name = '';
        this.versions = null;
    }
    static format(str) {
        return __awaiter(this, void 0, void 0, function* () {
            const githubSH = /^(github:)[-a-zA-Z0-9@:%._+~#=]+\/[-a-zA-Z0-9@:%._+~#=]+$/;
            const gitUrlRegexp = /((git|ssh|http(s)?)|(git@[\w.]+))(:(\/\/)?)([\w.@:/\-~]+)(\.git)(\/)?$/;
            let repo;
            if (isRelativePath(str) || isAbsolute(str)) {
                const localRepo = new LocalRepository(str);
                if (!(yield localRepo.existed())) {
                    throw new Error(`Template path cannot be found. Ensure it is an exist directory: ${localRepo.path}.`);
                }
                repo = localRepo;
            }
            else if (gitUrlRegexp.test(str)) {
                repo = new GitRepository(str);
            }
            else if (/^npm:/.test(str) && validateNpmPackageName(str.substring('npm:'.length))) {
                repo = new NpmRepository(str.substring('npm:'.length));
            }
            else if (githubSH.test(str)) {
                repo = new GitRepository(`https://github.com/${str.replace(/^github:/, '')}.git`);
            }
            else {
                throw new Error(`Invalid repository url: ${str}`);
            }
            if (!(yield repo.isVerioning()))
                logger.warn('The template repository is not versioned.');
            yield repo.checkout('latest');
            return repo;
        });
    }
    hasVersion(version) {
        return __awaiter(this, void 0, void 0, function* () {
            const versions = yield this.getVersions();
            return versions.includes(version);
        });
    }
    checkout(version = 'latest') {
        return __awaiter(this, void 0, void 0, function* () {
            if (version !== 'latest' && version !== 'default' && !semver.valid(version)) {
                throw new Error('Semantic version expected.');
            }
            const versions = yield this.getVersions();
            if (version === 'default') {
                this.version = 'default';
            }
            else if (version === 'latest') {
                if (versions.length)
                    this.version = versions[0];
                else
                    this.version = undefined;
            }
            else if (versions.includes(version)) {
                this.version = version;
            }
            else {
                throw new Error(`Cannot find template for the version ${version} `);
            }
        });
    }
    isVerioning() {
        return __awaiter(this, void 0, void 0, function* () {
            const versions = yield this.getVersions();
            if (versions.length)
                return true;
            return false;
        });
    }
    isLatest() {
        return __awaiter(this, void 0, void 0, function* () {
            const versions = yield this.getVersions();
            if (versions.length && this.version === versions[0])
                return true;
            return false;
        });
    }
    installDeps() {
        return __awaiter(this, void 0, void 0, function* () {
            const { storage } = this;
            const npmConfigFile = join(storage, 'package.json');
            const npmConfigFileExist = yield Effect.fs.pathExists(npmConfigFile);
            if (npmConfigFileExist)
                yield installDeps(storage);
        });
    }
    install(options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const { version } = this;
            if (version && !this.hasVersion(version)) {
                throw new Error(`The template version ${version} is not existed.`);
            }
            yield this.download();
            if (!options.noDeps)
                yield this.installDeps();
            return Template.load(this);
        });
    }
}
