var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import Ajv from 'ajv';
import ajvKeywords from 'ajv-keywords';
import { join, dirname } from 'path';
import { cosmiconfig } from 'cosmiconfig';
import { unnest, clone } from 'ramda';
import { loadNpmConfig } from "../loader";
import { TemplateSchema, RuleSchema, QuestionSchema, HandlerSchema } from "../schema";
import { isDirectory, logger } from "../utils";
import { UpgradeType } from "../consts";
import { Repository, Hook, Hooks, Listener, Resource, File, Files, CheckOptions, Question, Questions, Rule, Effect, CompiledFile, CompiledFiles, } from "../internal";
const ajv = new Ajv({ useDefaults: true, $data: true });
ajvKeywords(ajv);
const validateTemplateConfig = ajv
    .addSchema([HandlerSchema])
    .addSchema([RuleSchema, QuestionSchema])
    .compile(TemplateSchema);
export class Template {
    constructor(repo, path, engines, files, questions = [], hooks = []) {
        this.path = path;
        this.repository = repo;
        this.engines = engines;
        this.files = files;
        this.questions = questions;
        this.hooks = hooks;
    }
    compile(resource) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.info('compile template');
            const promises = this.files.map(file => file.compile(resource));
            return yield Promise.all(promises);
        });
    }
    ensureFolder(folders, prefix = '') {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = Object.entries(folders)
                .map(([name, child]) => __awaiter(this, void 0, void 0, function* () {
                const dir = join(prefix, name);
                yield Effect.fs.ensureDir(dir);
                yield this.ensureFolder(child, dir);
            }));
            yield Promise.all(promises);
        });
    }
    ensureDir(files) {
        return __awaiter(this, void 0, void 0, function* () {
            const folders = {};
            files.forEach(file => {
                const dir = dirname(file.projectPath);
                let parent = folders;
                dir.split('/').forEach(name => {
                    const pair = name || '/';
                    let next = parent[pair];
                    if (!next)
                        next = {};
                    parent[pair] = next;
                    parent = next;
                });
            });
            yield this.ensureFolder(folders);
        });
    }
    render(resource) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = yield this.compile(resource);
            logger.info('rendering');
            yield this.ensureDir(files);
            const promises = files.map(file => file.render());
            yield Promise.all(promises);
        });
    }
    check(resource, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = yield this.compile(resource);
            const errors = [];
            const promises = files.map((file) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield file.check(options);
                }
                catch (e) {
                    errors.push(e.message);
                }
            }));
            yield Promise.all(promises);
            if (errors.length) {
                errors.forEach(message => logger.error(message));
                throw new Error('Check failed');
            }
        });
    }
    static load(repo) {
        return __awaiter(this, void 0, void 0, function* () {
            const cwd = repo.storage;
            let entry = join(cwd, 'index.js');
            try {
                const npmConfig = yield loadNpmConfig(cwd);
                if (npmConfig.main)
                    entry = join(cwd, npmConfig.main);
            }
            catch (e) {
            }
            /**
             * NOTE: don't change the object that return by cosmiconfig.
             *       It was cache by cosmiconfig
             */
            const result = yield cosmiconfig('template').load(entry);
            if (!result) {
                throw new Error([
                    'Cannot load template config',
                    `Maybe syntax error in ${entry}`,
                ].join('\n'));
            }
            const config = clone(result.config);
            const valid = validateTemplateConfig(config);
            if (!valid) {
                throw new Error([
                    'There is some error in template config: ',
                    ajv.errorsText(validateTemplateConfig.errors, { dataVar: 'template' }),
                ].join('\n'));
            }
            /** absoult template path */
            const path = join(cwd, config.path);
            /** generate files */
            const rules = config.rules
                .map(item => (Object.assign(Object.assign({}, item), { path: join(path, item.path) })))
                .map(item => Rule.format(item));
            const rootRule = new Rule(path, UpgradeType.Cover, true);
            const files = yield this.searchDirFile(path, rootRule, rules);
            /** generate questions */
            let questions = [];
            if (config.questions)
                questions = config.questions.map(question => new Question(question));
            /** generate hooks */
            let hooks = [];
            if (config.hooks) {
                hooks = Object.entries(config.hooks)
                    .map(([name, listener]) => new Hook(name, listener));
            }
            return new Template(repo, path, config.engines, files, questions, hooks);
        });
    }
    static searchDirFile(path, rule, rules) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = yield Effect.fs.readdir(path);
            const promises = files.map((filename) => __awaiter(this, void 0, void 0, function* () {
                const subPath = join(path, filename);
                let subRule = rules.find(rule => rule.match(subPath));
                if (!subRule)
                    subRule = rule;
                else
                    subRule = Rule.merge(rule, subRule);
                if (yield isDirectory(subPath)) {
                    return yield this.searchDirFile(subPath, subRule, rules);
                }
                return subRule.createFile(subPath);
            }));
            const subFiles = yield Promise.all(promises);
            return unnest(subFiles);
        });
    }
}
