"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const semver_1 = __importDefault(require("semver"));
const promise_1 = __importDefault(require("simple-git/promise"));
const repository_1 = require("./repository");
const internal_1 = require("../../internal");
const utils_1 = require("../../utils");
const consts_1 = require("../../consts");
const path_1 = require("path");
const gitUrlRegexp = /((git|ssh|http(s)?)|(git@[\w.]+))(:(\/\/)?)([\w.@:/\-~]+)(\.git)(\/)?$/;
class GitRepository extends repository_1.Repository {
    constructor(url) {
        super();
        this.type = 'git';
        this.url = url;
        const matched = url.match(gitUrlRegexp);
        const [, , , , , , , links] = matched;
        const [owner, name] = links.split('/').slice(-2);
        this.owner = owner;
        this.name = name;
    }
    get record() {
        return this.url;
    }
    get storage() {
        const { url, version } = this;
        return path_1.join(consts_1.TEMPLATE_STORAGE, encodeURIComponent(url), version || 'noversion');
    }
    getVersions() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.versions)
                return this.versions;
            yield internal_1.Effect.fs.ensureDir(this.storage);
            const result = yield promise_1.default(this.storage).listRemote(['--tags', this.url]);
            const arr = result.split('\n');
            const versions = arr
                .filter(item => item.length && !/\^{}$/.test(item))
                .map(item => {
                const [, ref] = item.split(/\s+/);
                const number = ref.substring('refs/tags/v'.length);
                return number;
            })
                .filter(version => semver_1.default.valid(version))
                .sort((v1, v2) => semver_1.default.rcompare(v1, v2));
            if (!versions.length) {
                utils_1.logger.warn([
                    'Cannot get template versions, May be caused by the following reasons:',
                    `1. repository is not a mili template(${this.url})`,
                    '2. template have not a valid tag to mark the version(e.g. v1.0.0)',
                    `3. cannot get versions by command: \`git ls-remote --tags ${this.url}}\``,
                ].join('\n'));
            }
            this.versions = versions;
            return versions;
        });
    }
    download() {
        return __awaiter(this, void 0, void 0, function* () {
            const { url, version, storage } = this;
            if (!version || version === 'default') {
                if (!version)
                    utils_1.logger.warn('Version is unset, use the default branch files of git repository');
                yield internal_1.Effect.fs.remove(storage);
                yield internal_1.Effect.fs.ensureDir(storage);
                yield promise_1.default(storage).clone(url, storage);
                return;
            }
            const repositoryExisted = yield internal_1.Effect.fs.pathExists(storage);
            if (!repositoryExisted) {
                utils_1.logger.info(`clone template from ${url}...`);
                yield internal_1.Effect.fs.ensureDir(storage);
                yield promise_1.default(storage).clone(url, storage, ['--branch', `v${version}`, '--single-branch']);
                utils_1.logger.info(`template version: ${version}`);
            }
            else {
                utils_1.logger.info('use the cache of template');
            }
        });
    }
    existed() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield internal_1.Effect.fs.ensureDir(this.storage);
                const result = yield promise_1.default(this.storage).listRemote([]);
                return Boolean(result && result.length);
            }
            catch (e) {
                return false;
            }
        });
    }
}
exports.GitRepository = GitRepository;
