"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const validate_npm_package_name_1 = __importDefault(require("validate-npm-package-name"));
const utils_1 = require("../../utils");
const internal_1 = require("../../internal");
const semver_1 = __importDefault(require("semver"));
class Repository {
    constructor() {
        this.owner = '';
        this.name = '';
        this.versions = null;
    }
    static format(str) {
        return __awaiter(this, void 0, void 0, function* () {
            const githubSH = /^(github:)[-a-zA-Z0-9@:%._+~#=]+\/[-a-zA-Z0-9@:%._+~#=]+$/;
            const gitUrlRegexp = /((git|ssh|http(s)?)|(git@[\w.]+))(:(\/\/)?)([\w.@:/\-~]+)(\.git)(\/)?$/;
            let repo;
            if (utils_1.isRelativePath(str) || path_1.isAbsolute(str)) {
                const localRepo = new internal_1.LocalRepository(str);
                if (!(yield localRepo.existed())) {
                    throw new Error(`Template path cannot be found. Ensure it is an exist directory: ${localRepo.path}.`);
                }
                repo = localRepo;
            }
            else if (gitUrlRegexp.test(str)) {
                repo = new internal_1.GitRepository(str);
            }
            else if (/^npm:/.test(str) && validate_npm_package_name_1.default(str.substring('npm:'.length))) {
                repo = new internal_1.NpmRepository(str.substring('npm:'.length));
            }
            else if (githubSH.test(str)) {
                repo = new internal_1.GitRepository(`https://github.com/${str.replace(/^github:/, '')}.git`);
            }
            else {
                throw new Error(`Invalid repository url: ${str}`);
            }
            if (!(yield repo.isVerioning()))
                utils_1.logger.warn('The template repository is not versioned.');
            yield repo.checkout('latest');
            return repo;
        });
    }
    hasVersion(version) {
        return __awaiter(this, void 0, void 0, function* () {
            const versions = yield this.getVersions();
            return versions.includes(version);
        });
    }
    checkout(version = 'latest') {
        return __awaiter(this, void 0, void 0, function* () {
            if (version !== 'latest' && version !== 'default' && !semver_1.default.valid(version)) {
                throw new Error('Semantic version expected.');
            }
            const versions = yield this.getVersions();
            if (version === 'default') {
                this.version = 'default';
            }
            else if (version === 'latest') {
                if (versions.length)
                    this.version = versions[0];
                else
                    this.version = undefined;
            }
            else if (versions.includes(version)) {
                this.version = version;
            }
            else {
                throw new Error(`Cannot find template for the version ${version} `);
            }
        });
    }
    isVerioning() {
        return __awaiter(this, void 0, void 0, function* () {
            const versions = yield this.getVersions();
            if (versions.length)
                return true;
            return false;
        });
    }
    isLatest() {
        return __awaiter(this, void 0, void 0, function* () {
            const versions = yield this.getVersions();
            if (versions.length && this.version === versions[0])
                return true;
            return false;
        });
    }
    installDeps() {
        return __awaiter(this, void 0, void 0, function* () {
            const { storage } = this;
            const npmConfigFile = path_1.join(storage, 'package.json');
            const npmConfigFileExist = yield internal_1.Effect.fs.pathExists(npmConfigFile);
            if (npmConfigFileExist)
                yield utils_1.installDeps(storage);
        });
    }
    install(options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const { version } = this;
            if (version && !this.hasVersion(version)) {
                throw new Error(`The template version ${version} is not existed.`);
            }
            yield this.download();
            if (!options.noDeps)
                yield this.installDeps();
            return internal_1.Template.load(this);
        });
    }
}
exports.Repository = Repository;
