"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ajv_1 = __importDefault(require("ajv"));
const micromatch_1 = __importDefault(require("micromatch"));
const consts_1 = require("../consts");
const schema_1 = require("../schema");
const infer_encoding_1 = require("../infer-encoding");
const internal_1 = require("../internal");
const path_1 = require("path");
const ajv = new ajv_1.default({ useDefaults: true });
const validate = ajv
    .addSchema([schema_1.HandlerSchema])
    .compile(schema_1.RuleSchema);
class Rule {
    constructor(path, upgrade = consts_1.UpgradeType.Cover, glob = true, inferEncoding = infer_encoding_1.inferEncodingNormally, handler) {
        this.path = path;
        this.inferEncoding = inferEncoding;
        this.upgrade = upgrade;
        this.glob = glob;
        this.handler = handler;
    }
    static format(obj) {
        const valid = validate(obj);
        if (!valid) {
            throw new TypeError([
                'Incorrect rules field configuration for template configuration',
                ajv.errorsText(validate.errors, { dataVar: 'rule' }),
            ].join('\n'));
        }
        if (!path_1.isAbsolute(obj.path))
            throw new TypeError(`The path of rule should be absolute path. But get ${obj.path}`);
        const path = obj.path;
        const upgrade = obj.upgrade;
        const glob = obj.glob;
        let inferEncoding = infer_encoding_1.inferEncodingNormally;
        if ('encoding' in obj) {
            if (typeof obj.encoding === 'object') {
                inferEncoding = infer_encoding_1.inferEncodingByMapping(obj.encoding);
            }
            else if (Object.values(consts_1.Encoding).includes(obj.encoding)) {
                inferEncoding = infer_encoding_1.inferEncodingImmobile(obj.encoding);
            }
        }
        let handler;
        if (obj.handlers) {
            const handlers = obj.handlers.map(internal_1.Handler.format);
            handler = internal_1.Handler.compose(handlers);
        }
        else if (obj.handler) {
            handler = internal_1.Handler.format(obj.handler);
        }
        return new Rule(path, upgrade, glob, inferEncoding, handler);
    }
    match(path) {
        if (this.glob)
            return micromatch_1.default.isMatch(path, this.path);
        return this.path === path;
    }
    static merge(parent, child) {
        const path = child.path;
        const upgrade = child.upgrade;
        const glob = child.glob;
        const handlers = [];
        let handler;
        if (parent.handler)
            handlers.push(parent.handler);
        if (child.handler)
            handlers.push(child.handler);
        if (handlers.length)
            handler = internal_1.Handler.compose(handlers);
        const inferEncoding = child.inferEncoding || parent.inferEncoding;
        return new Rule(path, upgrade, glob, inferEncoding, handler);
    }
    createFile(path) {
        const paths = path.split('/')
            .map((pair, i, arr) => arr.slice(0, arr.length - i).join('/'))
            .filter(item => Boolean(item));
        if (!paths.some(item => this.match(item))) {
            throw new Error([
                'Cannot create file from rule',
                'Because the file path is not match rule',
            ].join('\n'));
        }
        const { inferEncoding, upgrade } = this;
        let handler = this.handler;
        let upgradeHandler;
        if (upgrade === 'merge')
            upgradeHandler = internal_1.buildInHandlers.merge;
        if (upgrade === 'exist')
            upgradeHandler = internal_1.buildInHandlers.exist;
        if (upgrade === 'keep')
            upgradeHandler = internal_1.buildInHandlers.ignoreWhen(({ operation }) => operation !== 'init');
        if (handler && upgradeHandler)
            handler = internal_1.Handler.compose([handler, upgradeHandler]);
        else if (!handler && upgradeHandler)
            handler = upgradeHandler;
        return new internal_1.File(path, inferEncoding, handler);
    }
}
exports.Rule = Rule;
