"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ajv_1 = __importDefault(require("ajv"));
const ajv_keywords_1 = __importDefault(require("ajv-keywords"));
const path_1 = require("path");
const cosmiconfig_1 = require("cosmiconfig");
const ramda_1 = require("ramda");
const loader_1 = require("../loader");
const schema_1 = require("../schema");
const utils_1 = require("../utils");
const consts_1 = require("../consts");
const internal_1 = require("../internal");
const ajv = new ajv_1.default({ useDefaults: true, $data: true });
ajv_keywords_1.default(ajv);
const validateTemplateConfig = ajv
    .addSchema([schema_1.HandlerSchema])
    .addSchema([schema_1.RuleSchema, schema_1.QuestionSchema])
    .compile(schema_1.TemplateSchema);
class Template {
    constructor(repo, path, engines, files, questions = [], hooks = []) {
        this.path = path;
        this.repository = repo;
        this.engines = engines;
        this.files = files;
        this.questions = questions;
        this.hooks = hooks;
    }
    compile(resource) {
        return __awaiter(this, void 0, void 0, function* () {
            utils_1.logger.info('compile template');
            const promises = this.files.map(file => file.compile(resource));
            return yield Promise.all(promises);
        });
    }
    ensureFolder(folders, prefix = '') {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = Object.entries(folders)
                .map(([name, child]) => __awaiter(this, void 0, void 0, function* () {
                const dir = path_1.join(prefix, name);
                yield internal_1.Effect.fs.ensureDir(dir);
                yield this.ensureFolder(child, dir);
            }));
            yield Promise.all(promises);
        });
    }
    ensureDir(files) {
        return __awaiter(this, void 0, void 0, function* () {
            const folders = {};
            files.forEach(file => {
                const dir = path_1.dirname(file.projectPath);
                let parent = folders;
                dir.split('/').forEach(name => {
                    const pair = name || '/';
                    let next = parent[pair];
                    if (!next)
                        next = {};
                    parent[pair] = next;
                    parent = next;
                });
            });
            yield this.ensureFolder(folders);
        });
    }
    render(resource) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = yield this.compile(resource);
            utils_1.logger.info('rendering');
            yield this.ensureDir(files);
            const promises = files.map(file => file.render());
            yield Promise.all(promises);
        });
    }
    check(resource, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = yield this.compile(resource);
            const errors = [];
            const promises = files.map((file) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield file.check(options);
                }
                catch (e) {
                    errors.push(e.message);
                }
            }));
            yield Promise.all(promises);
            if (errors.length) {
                errors.forEach(message => utils_1.logger.error(message));
                throw new Error('Check failed');
            }
        });
    }
    static load(repo) {
        return __awaiter(this, void 0, void 0, function* () {
            const cwd = repo.storage;
            let entry = path_1.join(cwd, 'index.js');
            try {
                const npmConfig = yield loader_1.loadNpmConfig(cwd);
                if (npmConfig.main)
                    entry = path_1.join(cwd, npmConfig.main);
            }
            catch (e) {
            }
            /**
             * NOTE: don't change the object that return by cosmiconfig.
             *       It was cache by cosmiconfig
             */
            const result = yield cosmiconfig_1.cosmiconfig('template').load(entry);
            if (!result) {
                throw new Error([
                    'Cannot load template config',
                    `Maybe syntax error in ${entry}`,
                ].join('\n'));
            }
            const config = ramda_1.clone(result.config);
            const valid = validateTemplateConfig(config);
            if (!valid) {
                throw new Error([
                    'There is some error in template config: ',
                    ajv.errorsText(validateTemplateConfig.errors, { dataVar: 'template' }),
                ].join('\n'));
            }
            /** absoult template path */
            const path = path_1.join(cwd, config.path);
            /** generate files */
            const rules = config.rules
                .map(item => (Object.assign(Object.assign({}, item), { path: path_1.join(path, item.path) })))
                .map(item => internal_1.Rule.format(item));
            const rootRule = new internal_1.Rule(path, consts_1.UpgradeType.Cover, true);
            const files = yield this.searchDirFile(path, rootRule, rules);
            /** generate questions */
            let questions = [];
            if (config.questions)
                questions = config.questions.map(question => new internal_1.Question(question));
            /** generate hooks */
            let hooks = [];
            if (config.hooks) {
                hooks = Object.entries(config.hooks)
                    .map(([name, listener]) => new internal_1.Hook(name, listener));
            }
            return new Template(repo, path, config.engines, files, questions, hooks);
        });
    }
    static searchDirFile(path, rule, rules) {
        return __awaiter(this, void 0, void 0, function* () {
            const files = yield internal_1.Effect.fs.readdir(path);
            const promises = files.map((filename) => __awaiter(this, void 0, void 0, function* () {
                const subPath = path_1.join(path, filename);
                let subRule = rules.find(rule => rule.match(subPath));
                if (!subRule)
                    subRule = rule;
                else
                    subRule = internal_1.Rule.merge(rule, subRule);
                if (yield utils_1.isDirectory(subPath)) {
                    return yield this.searchDirFile(subPath, subRule, rules);
                }
                return subRule.createFile(subPath);
            }));
            const subFiles = yield Promise.all(promises);
            return ramda_1.unnest(subFiles);
        });
    }
}
exports.Template = Template;
